package com.oracle.net;

import java.net.SocketException;
import java.nio.channels.ServerSocketChannel;
import sun.nio.ch.Secrets;
import sun.net.sdp.SdpSupport;
import java.nio.channels.SocketChannel;
import java.io.IOException;
import java.net.Socket;
import java.lang.reflect.InvocationTargetException;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.reflect.AccessibleObject;
import java.net.SocketImpl;
import java.net.ServerSocket;
import java.lang.reflect.Constructor;

public final class Sdp
{
    private static final Constructor<ServerSocket> serverSocketCtor;
    private static final Constructor<SocketImpl> socketImplCtor;
    
    private Sdp() {
    }
    
    private static void setAccessible(final AccessibleObject accessibleObject) {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                accessibleObject.setAccessible(true);
                return null;
            }
        });
    }
    
    private static SocketImpl createSocketImpl() {
        try {
            return (SocketImpl)Sdp.socketImplCtor.newInstance(new Object[] { Boolean.valueOf(true) });
        }
        catch (InstantiationException detailMessage) {
            throw new AssertionError(detailMessage);
        }
        catch (IllegalAccessException detailMessage2) {
            throw new AssertionError(detailMessage2);
        }
        catch (InvocationTargetException detailMessage3) {
            throw new AssertionError(detailMessage3);
        }
    }
    
    public static Socket openSocket() throws IOException {
        return new Sdp.SdpSocket(createSocketImpl());
    }
    
    public static ServerSocket openServerSocket() throws IOException {
        SocketImpl socketImpl = createSocketImpl();
        try {
            return (ServerSocket)Sdp.serverSocketCtor.newInstance(new Object[] { socketImpl });
        }
        catch (IllegalAccessException detailMessage) {
            throw new AssertionError(detailMessage);
        }
        catch (InstantiationException detailMessage2) {
            throw new AssertionError(detailMessage2);
        }
        catch (InvocationTargetException cause) {
            Throwable cause2 = cause.getCause();
            if ((cause2 instanceof IOException)) {
                throw (IOException)cause2;
            }
            if ((cause2 instanceof RuntimeException)) {
                throw (RuntimeException)cause2;
            }
            throw new RuntimeException(cause);
        }
    }
    
    public static SocketChannel openSocketChannel() throws IOException {
        return Secrets.newSocketChannel(SdpSupport.createSocket());
    }
    
    public static ServerSocketChannel openServerSocketChannel() throws IOException {
        return Secrets.newServerSocketChannel(SdpSupport.createSocket());
    }
    
    static {
        try {
            setAccessible(serverSocketCtor = ServerSocket.class.getDeclaredConstructor(new Class[] { SocketImpl.class }));
        }
        catch (NoSuchMethodException detailMessage) {
            throw new AssertionError(detailMessage);
        }
        try {
            setAccessible(socketImplCtor = Class.forName("java.net.SdpSocketImpl", true, null).getDeclaredConstructor(new Class[] { Boolean.TYPE }));
        }
        catch (ClassNotFoundException detailMessage2) {
            throw new AssertionError(detailMessage2);
        }
        catch (NoSuchMethodException detailMessage3) {
            throw new AssertionError(detailMessage3);
        }
    }
    
    private static class SdpSocket extends Socket
    {
        SdpSocket(SocketImpl impl) throws SocketException {
            super(impl);
        }
    }
}
